searchNodes=[{"doc":"Execute and inline expressions at compile time. In a regular code, all you need is Exp.inline/1 . For usage inside of macros when you don't know if arguments are safe, see Exp.maybe_inline/1 . Exp.abs/1 is a real-world example of using Exp.maybe_inline/1 . Exp.pure_func?/3 and Exp.safe_node?/1 are utility function that Exp.maybe_inline/1 uses to make decision if expression is safe to inline. Source: github.com/orsinium-labs/exp","ref":"Exp.html","title":"Exp","type":"module"},{"doc":"Inlined version of Kernel.abs/1 . Returns the arithmetical absolute value of the number . Examples iex&gt; Exp . abs ( - 2 ) 2 iex&gt; Exp . abs ( 2 ) 2 This is how you can check if it was inlined: iex&gt; # inlined: iex&gt; q = quote do : Exp . abs ( - 2 ) iex&gt; 2 = Macro . expand ( q , __ENV__ ) iex&gt; # not inlined because unsafe: iex&gt; q = quote do : Exp . abs ( node ( ) ) iex&gt; { :abs , _ , _ } = Macro . expand ( q , __ENV__ )","ref":"Exp.html#abs/1","title":"Exp.abs/1","type":"macro"},{"doc":"Macro to statically execute and inline an expression at compile time. If the expression cannot be inlined, it will explode at compile time. This is your responsibility to make sure the code can be inlined and is safe to inline. What to inline There are requirements for a good candidate to be inlined: It doesn't make network requests. It doesn't depend on a service or ecto. For the same input, it always produces the same output. It's not too slow. Its result doesn't take too much memory. A good example of such function is Regex.compile!/2 . Examples The usage is straightforward, just wrap whatever you want to inline: iex&gt; require Exp iex&gt; Exp . inline ( 1 + 2 ) 3 The difference is that the expression inside will be executed at compile time (when expanding the AST) and included into the compiled code: iex(21)&gt; q = quote do : 1 + 2 iex(22)&gt; { :+ , _ , [ 1 , 2 ] } = Macro . expand ( q , __ENV__ ) iex(23)&gt; q = quote do : Exp . inline ( 1 + 2 ) iex(24)&gt; 3 = Macro . expand ( q , __ENV__ )","ref":"Exp.html#inline/1","title":"Exp.inline/1","type":"macro"},{"doc":"A safe implementation of inline/1 for macros. It inlines the given quoted expression if and only if all unquote arguments are safe to execute at compile time. The safety of the expression itself isn't checked. The result is also a quoted expression. It's supposed to be used from macros when you don't know if the macros will be used for a safe to execute code or not. Examples Here, var is safe and so the expression is inlined: iex&gt; var = quote do : 13 iex&gt; 13 = Exp . maybe_inline ( quote do : abs ( unquote ( var ) ) ) Here, var is not safe and so the expression is left as-is: iex&gt; var = quote do : node ( ) iex&gt; { :abs , _ , _ } = Exp . maybe_inline ( quote do : abs ( unquote ( var ) ) ) Here, the expression isn't safe but it's still inlined because maybe_inline/2 checks only safety of expressions inside Kernel.SpecialForms.unquote/1 : iex&gt; Exp . maybe_inline ( quote do : node ( ) ) :nonode@nohost See also Exp.abs/1 for a real-world usage example.","ref":"Exp.html#maybe_inline/1","title":"Exp.maybe_inline/1","type":"macro"},{"doc":"Check if the function of given name and arity is pure. Examples iex&gt; Exp . pure_func? ( :abs , 1 ) true iex&gt; Exp . pure_func? ( :abs , 2 ) # bad arity false iex&gt; Exp . pure_func? ( :send , 2 ) false","ref":"Exp.html#pure_func?/2","title":"Exp.pure_func?/2","type":"function"},{"doc":"Check if the function of given module, name, and arity is pure. Examples iex&gt; Exp . pure_func? ( String , :upcase , 1 ) true iex&gt; Exp . pure_func? ( String , :upcase , 4 ) # bad arity false iex&gt; Exp . pure_func? ( File , :cwd , 0 ) false iex&gt; Exp . pure_func? ( Kernel , :abs , 1 ) true iex&gt; Exp . pure_func? ( :erlang , :abs , 1 ) true","ref":"Exp.html#pure_func?/3","title":"Exp.pure_func?/3","type":"function"},{"doc":"Check if the given AST node is safe to be statically executed. This check is conservative. If it doesn't know anything about the function, the result is false . It's used be Exp.maybe_inline/1 to make a decision if the code should be inlined. Examples iex&gt; Exp . safe_node? ( quote do : 1 ) true iex&gt; Exp . safe_node? ( quote do : &quot;hello&quot; ) true iex&gt; Exp . safe_node? ( quote do : 1 + 2 ) true iex&gt; Exp . safe_node? ( quote do : div ( 2 , 3 ) ) true iex&gt; Exp . safe_node? ( quote do : File . cwd ( ) ) false","ref":"Exp.html#safe_node?/1","title":"Exp.safe_node?/1","type":"function"}]